var gulp = require('gulp')
var babel = require('gulp-babel')
var sourcemaps = require('gulp-sourcemaps')
var glob = require('glob')
var path = require('path')
var fs = require('fs')

var bundleFileName = 'bundle'

gulp.task('bundle', function (done) {
  var files = glob.sync('./src/**/*.js')
  var fileNames = files.map(function (file) {
    return path.basename(file, '.js')
  }).filter(function (filename) {
    return filename !== bundleFileName
  })

  var imports = fileNames.reduce(function (acc, fileName) {
    acc += `\nimport ${fileName} from "./${fileName}"`
    return acc
  }, '')

  var exports = fileNames.reduce(function (acc, fileName) {
    acc += `\nexport { ${fileName} }`
    return acc
  }, '')

  var jsonPatchUtils = '\nvar jsonPatchUtils = {}'
  jsonPatchUtils += fileNames.reduce(function (acc, fileName) {
    acc += `\njsonPatchUtils.${fileName} = ${fileName}`
    return acc
  }, '')
  jsonPatchUtils += '\nexport default jsonPatchUtils'

  var bundle = imports + jsonPatchUtils + exports

  fs.writeFileSync(`./src/${bundleFileName}.js`, bundle, 'utf8')

  done()
})

gulp.task('compile', function () {
  return gulp.src('src/**/*')
    .pipe(sourcemaps.init())
    .pipe(babel({
      presets: ['es2015']
    }))
    .pipe(sourcemaps.write('.'))
    .pipe(gulp.dest('dist/'))
})

gulp.task('build', ['bundle', 'compile'])
